from ringo.lib.helpers import get_item_modul
from ringo.lib.renderer.form import ListingFieldRenderer


class TagFieldRenderer(ListingFieldRenderer):
    """Renderer to render tags  listings. The renderer will only show
    tags which are either associated to no module or the the items
    module."""

    def __init__(self, field, translate):
        ListingFieldRenderer.__init__(self, field, translate)

    def _get_all_items(self):
        tags = []
        alltags = ListingFieldRenderer._get_all_items(self)
        item_modul = get_item_modul(self._field._form._request,
                                    self._field._form._item).id
        for tag in alltags:
            if not tag.modul or (tag.modul.id == item_modul):
                tags.append(tag)
        alltags.items = tags
        return alltags



import cgi
import sqlalchemy as sa
from sqlalchemy.ext.declarative import declared_attr
from ringo.lib.helpers import literal
from ringo.model import Base
from ringo.model.base import BaseItem, BaseFactory
from ringo.model.mixins import Owned


class TagFactory(BaseFactory):

    def create(self, user=None, values=None):
        new_item = BaseFactory.create(self, user, values)
        return new_item


class Tag(BaseItem, Owned, Base):
    """Tags (keywords) can be used to mark items."""

    __tablename__ = 'tags'
    """Name of the table in the database for this modul. Do not
    change!"""
    _modul_id = None
    """Will be set dynamically. See include me of this modul"""

    # Define columns of the table in the database
    id = sa.Column(sa.Integer, primary_key=True)
    name = sa.Column('name', sa.Text, default=None)
    description = sa.Column('description', sa.Text, default=None)
    tagtype = sa.Column('type', sa.Integer, default=None)

    # Relation to a modul. Tags can be assigned to a module for
    # filtering.
    mid = sa.Column(sa.Integer, sa.ForeignKey('modules.id'))
    modul = sa.orm.relationship("ModulItem", backref="tags")

    @classmethod
    def get_item_factory(cls):
        return TagFactory(cls)

    def render(self):
        mapping = {
            0: "label label-default",
            1: "label label-primary",
            2: "label label-success",
            3: "label label-info",
            4: "label label-warning",
            5: "label label-danger"}
        return literal('<span class="%s">%s</span>'
                       % (mapping.get(self.tagtype), cgi.escape(self.name)))


class Tagged(object):
    """Mixin to add tag (keyword) functionallity to a modul. Adding this Mixin
    the item of a modul will have a "tags" relationship containing all
    the tag entries for this item."""

    @declared_attr
    def tags(cls):
        tbl_name = "nm_%s_tags" % cls.__name__.lower()
        nm_table = sa.Table(tbl_name, Base.metadata,
                            sa.Column('iid', sa.Integer,
                                      sa.ForeignKey(cls.id)),
                            sa.Column('tid', sa.Integer,
                                      sa.ForeignKey("tags.id")))
        tags = sa.orm.relationship(Tag, secondary=nm_table)
        return tags



#!/usr/bin/env python
# encoding: utf-8

from setuptools import setup, find_packages
import sys, os

version = '1.0.2'

setup(name='ringo_tag',
      version=version,
      description="Tag extention for the ringo webframework",
      long_description="""""",
      classifiers=[], # Get strings from http://pypi.python.org/pypi?%3Aaction=list_classifiers
      keywords='ringo pyramid extension',
      author='Torsten Irländer',
      author_email='',
      url='',
      license='GPL v2 or later',
      packages=find_packages(exclude=['ez_setup', 'examples', 'tests']),
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'ringo'
      ],
      entry_points="""
      # -*- Entry points: -*-
      [babel.extractors]
      tableconfig = ringo.lib.i18n:extract_i18n_tableconfig
      formconfig = formbar.i18n:extract_i18n_formconfig
      """,
      message_extractors = {'ringo_tag': [
            ('**.py', 'python', None),
            ('templates/**.html', 'mako', None),
            ('templates/**.mako', 'mako', None),
            ('**.xml', 'formconfig', None),
            ('**.json', 'tableconfig', None),
            ('static/**', 'ignore', None)]},
      )



import logging
from pyramid.i18n import TranslationStringFactory
from ringo.lib.i18n import translators
from ringo.lib.extension import register_modul
from ringo.lib.helpers import dynamic_import

# Import models so that alembic is able to autogenerate migrations
# scripts.
from ringo_tag.model import Tag

log = logging.getLogger(__name__)

modul_config = {
    "name": "tag",
    "label": "Tag",
    "label_plural": "Tags",
    "clazzpath": "ringo_tag.model.Tag",
    "str_repr": "%s|name",
    "display": "admin-menu",
    "actions": ["list", "read", "update", "create", "delete"]
}


def includeme(config):
    """Registers a new modul for ringo.

    :config: Dictionary with configuration of the new modul

    """
    modul = register_modul(config, modul_config)
    Tag._modul_id = modul.get_value("id")
    translators.append(TranslationStringFactory('ringo_tag'))
    config.add_translation_dirs('ringo_tag:locale/')
